package com.qmth.teachcloud.cas.mq.gdpu.api;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.teachcloud.cas.mq.gdpu.bean.AESEncryptUtils;
import com.qmth.teachcloud.cas.mq.gdpu.bean.ProcessInfoDTO;
import com.qmth.teachcloud.cas.mq.gdpu.bean.RemarkInfoDTO;
import com.qmth.teachcloud.cas.mq.gdpu.bean.TodoInfoDTO;
import com.qmth.teachcloud.cas.mq.gdpu.cache.FlowMqCacheUtil;
import com.qmth.teachcloud.exchange.common.bean.dto.flow.DoneTaskDto;
import com.qmth.teachcloud.exchange.common.bean.dto.flow.StandardFlowWorkMsg;
import com.qmth.teachcloud.exchange.common.bean.dto.flow.ToDoTaskDto;
import com.qmth.teachcloud.exchange.common.bean.dto.syssetting.SimpleObject;
import com.qmth.teachcloud.exchange.common.contant.SystemConstant;
import com.qmth.teachcloud.exchange.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.exchange.common.service.CommonCacheService;
import com.qmth.teachcloud.exchange.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;

@Api(tags = "广东药科大学服务中台controller")
@RestController
//@RequestMapping(SystemConstant.PREFIX_URL_GDPU_GET_BACK_LOG)
@Aac(auth = BOOL.FALSE)
@Validated
public class GdpuMqController {
    private final static Logger log = LoggerFactory.getLogger(GdpuMqController.class);

    @Resource
    CommonCacheService commonCacheService;

    @RequestMapping(value = SystemConstant.PREFIX_URL_GDPU_GET_BACK_LOG, method = RequestMethod.POST)
    @ApiOperation(value = "获取流程事项信息")
    @ApiResponses({@ApiResponse(code = 200, message = "常规信息", response = ResultUtil.class)})
    public JSONObject syncAffairInfo(@ApiParam(value = "appid", required = true) @RequestHeader String appid,
                                     @ApiParam(value = "签名串", required = true) @RequestHeader String secret,
                                     @ApiParam(value = "流程id", required = true) @RequestHeader String orunId) {
        log.info("appid:{},secret:{},orunId:{}", appid, secret, orunId);
        if (Objects.isNull(appid) || Objects.equals(appid.trim(), "")) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("appid为空");
        }
        if (Objects.isNull(secret) || Objects.equals(secret.trim(), "")) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("secret为空");
        }
        if (Objects.isNull(orunId) || Objects.equals(orunId.trim(), "")) {
            throw ExceptionResultEnum.PARAMS_ERROR.exception("流水号为空");
        }
        Long flowId = null;
        if (Objects.nonNull(orunId)) {
            flowId = Long.parseLong(orunId.substring(4, orunId.length()));
        }
        Long schoolId = FlowMqCacheUtil.getSchoolId(flowId);
        Map<String, SimpleObject> mapSetting = commonCacheService.getSysSetting(schoolId);

        String appId = null, appSecret = null, sysAppId = null;
        if (!CollectionUtils.isEmpty(mapSetting)) {
            SimpleObject simpleObjectAppId = mapSetting.get(SystemConstant.GDPU_APP_ID_PARAM);
            Objects.requireNonNull(simpleObjectAppId, "未配置学校appId");

            SimpleObject simpleObjectAppSecret = mapSetting.get(SystemConstant.GDPU_APP_SECRET_PARAM);
            Objects.requireNonNull(simpleObjectAppSecret, "未配置学校secret");

            SimpleObject simpleObjectSysAppId = mapSetting.get(SystemConstant.GDPU_SYS_APP_ID_PARAM);
            Objects.requireNonNull(simpleObjectSysAppId, "未配置学校sysAppid");

            appId = simpleObjectAppId.getValue();
            appSecret = simpleObjectAppSecret.getValue();
            sysAppId = simpleObjectSysAppId.getValue();
        }
        //校验secret
        String dateStr = new SimpleDateFormat(SystemConstant.DEFAULT_DATE).format(new Date());
        String encryptedvalue = DigestUtils.md5Hex(appid + appSecret + dateStr).toUpperCase();
        JSONObject result = new JSONObject();

        JSONObject content = new JSONObject();
        if (!encryptedvalue.equals(secret)) {
            content.put("code", "501");
            content.put("message", "加密串校验失败!");
            result.put("content", content);
            return result;
        }
        JSONObject data = new JSONObject();
        ToDoTaskDto toDoTaskDto = FlowMqCacheUtil.getToDoTask(flowId);
        JSONArray processInfo = new JSONArray(); //主流程信息
        if (orunId != null) {
            ProcessInfoDTO processInfoDTO = new ProcessInfoDTO();
            processInfoDTO.setSysAppId(sysAppId);
            processInfoDTO.setOrunId(orunId);
            setProcessInfoDTO(processInfoDTO, toDoTaskDto);
            processInfo.add(processInfoDTO);
        }
        JSONArray todoInfo = new JSONArray(); //待办信息
        List<StandardFlowWorkMsg> todoTaskList = toDoTaskDto.getHandleMessageDto().getTodoTaskList();
        for (StandardFlowWorkMsg s : todoTaskList) {
            TodoInfoDTO todoInfoDTO = new TodoInfoDTO();
            setTodoInfoDTO(todoInfoDTO, s);
            todoInfo.add(todoInfoDTO);
        }
//        JSONArray toreadInfo = new JSONArray(); //待阅信息
//        if (orunId != null) {
//            ToReadInfoDTO toReadInfoDTO = new ToReadInfoDTO();
//            // TODO 根据业务系统和场景，传入待阅参数，如无数据，则无需传参。
//            setToReadInfoDTO(toReadInfoDTO);
//            toreadInfo.add(toReadInfoDTO);
//        }
        DoneTaskDto doneTaskDto = FlowMqCacheUtil.getDoneTask(flowId);
        JSONArray remarkInfo = new JSONArray(); //已办信息
        RemarkInfoDTO remarkInfoDTO = new RemarkInfoDTO();
        setRemarkInfoDTO(remarkInfoDTO, doneTaskDto);
        remarkInfo.add(remarkInfoDTO);

        data.put("processInfo", processInfo);
        data.put("todoInfo", todoInfo);
////        data.put("toreadInfo", toreadInfo);
//        data.put("remarkInfo", remarkInfo);
        content.put("statusCode", 200);
        content.put("success", true);
        content.put("message", "操作成功!");
        content.put("data", data);
        log.info("content:{}", content.toJSONString());
//请求参数body内容加密
        try {
            result.put("content", AESEncryptUtils.aesEncrypt(content.toJSONString(), appSecret.substring(0, 16)));
        } catch (Exception e) {
            System.out.println("AES加密失败");
            e.printStackTrace();
            throw new RuntimeException(e);
        }
        return result;
    }

    private void setProcessInfoDTO(ProcessInfoDTO processInfoDTO, ToDoTaskDto toDoTaskDto) {
//        processInfoDTO.setSysAppId("LYOA");
//        processInfoDTO.setOrunId("LYOA79ec0af89702d641d839d5bbb5e9073b0e4b");
        List<StandardFlowWorkMsg> standardFlowWorkMsgList = toDoTaskDto.getHandleMessageDto().getTodoTaskList();
        ;
        List<String> userIdList = new LinkedList<>(), userNameList = new LinkedList<>();
        for (StandardFlowWorkMsg s : standardFlowWorkMsgList) {
            userIdList.add(String.valueOf(s.getUserId()));
            userNameList.add(s.getUserName());
        }

        processInfoDTO.setSubject(toDoTaskDto.getFlowData().getName());
        processInfoDTO.setCurrentNodeId(String.valueOf(toDoTaskDto.getFlowData().getId()));
        processInfoDTO.setCurrentNodeName(toDoTaskDto.getFlowData().getName());
//        processInfoDTO.setStatus(standardFlowWorkMsgList.get(0).getStatus().name());
        processInfoDTO.setStatus("Current");
        processInfoDTO.setUserId(String.valueOf(toDoTaskDto.getFlowData().getCreateId())); //申请人ID
        processInfoDTO.setUserName(toDoTaskDto.getFlowData().getUpdateName()); //申请人名称
        processInfoDTO.setAuthor(StringUtils.join(userIdList, ",")); //当前审批人的账户ID（多个使用逗号分隔（示例：2007112, admin））
        processInfoDTO.setAuthorCn(StringUtils.join(userNameList, ",")); //当前审批人的账户名称（多个使用逗号分隔（示例：2007112, admin））
        processInfoDTO.setUrl("https://portal.lysky.com:6205/bpmui/formRender/run?docUnid=" + toDoTaskDto.getFlowData().getFlowId());
        processInfoDTO.setMobileUrl("https://portal.lysky.com:6205/bpmui/formRender/run?docUnid=" + toDoTaskDto.getFlowData().getFlowId());
        processInfoDTO.setDocAddTime(new Date());// "2022-07-25 02:36:00"
        processInfoDTO.setDesc(toDoTaskDto.getFlowData().getName());
    }

    private void setTodoInfoDTO(TodoInfoDTO todoInfoDTO, StandardFlowWorkMsg standardFlowWorkMsg) {
        todoInfoDTO.setUserId(String.valueOf(standardFlowWorkMsg.getUserId()));
        todoInfoDTO.setUserName(standardFlowWorkMsg.getUserName());
        todoInfoDTO.setStartTime(DateUtil.format(new Date(standardFlowWorkMsg.getCreateTime()), SystemConstant.DEFAULT_DATE_PATTERN));
        todoInfoDTO.setOrunId(String.valueOf(standardFlowWorkMsg.getExecutionId()));
        todoInfoDTO.setNodeId(String.valueOf(standardFlowWorkMsg.getTaskId()));
        todoInfoDTO.setNodeName(standardFlowWorkMsg.getTaskName());
        todoInfoDTO.setUrl("https://portal.lysky.com:6205/bpmui/#/formRender/run?docUnid=" + standardFlowWorkMsg.getFlowId());
        todoInfoDTO.setMobileUrl("https://portal.lysky.com:6205/bpmui/#/formRender/run?docUnid=" + standardFlowWorkMsg.getFlowId());
        todoInfoDTO.setStatus("Current"); //当出现或签、抢占流程时，需要将其他未办理用户设置为Pause状态。
    }

    //以审批通过为例，给出办理记录DTO的数据填充demo
    private void setRemarkInfoDTO(RemarkInfoDTO remarkInfoDTO, DoneTaskDto doneTaskDto) {
        remarkInfoDTO.setNodeId(String.valueOf(doneTaskDto.getTaskId()));
        remarkInfoDTO.setNodeName(doneTaskDto.getHistoricName());
        remarkInfoDTO.setStartTime(DateUtil.format(new Date(doneTaskDto.getCreateTime()), SystemConstant.DEFAULT_DATE_PATTERN));
        remarkInfoDTO.setEndTime(DateUtil.format(new Date(doneTaskDto.getCreateTime()), SystemConstant.DEFAULT_DATE_PATTERN));
        remarkInfoDTO.setActionName(doneTaskDto.getApproveOperation().name());
        remarkInfoDTO.setUserId(String.valueOf(doneTaskDto.getApproveId()));
        remarkInfoDTO.setUserName(doneTaskDto.getApproveUserName());
        remarkInfoDTO.setOrunId(String.valueOf(doneTaskDto.getId()));
        remarkInfoDTO.setRemark(doneTaskDto.getApproveRemark());
        remarkInfoDTO.setUrl("https://portal.lysky.com:6205/bpmui/#/formRender/run?docUnid=79ec0af89702d641d839d5bbb5e9073b0e4b");
        remarkInfoDTO.setMobileUrl("https://portal.lysky.com:6205/bpmui/#/formRender/run?docUnid=79ec0af89702d641d839d5bbb5e9073b0e4b");
    }

//    private void setToReadInfoDTO(ToReadInfoDTO toReadInfoDTO) {
//        JSONArray toReadUsers = new JSONArray();
////根据场景 -- 插入待阅用户记录
//        JSONObject toReadUser1 = new JSONObject();
//        toReadUser1.put("toReadUserId", "admin");
//        toReadUser1.put("toReadUserName", "超级管理员");
//        toReadUser1.put("Status", "Current");
//        toReadUser1.put("remark", null);
//        toReadUser1.put("startTime", "2022-12-09 17:34:46");
//        toReadUsers.add(toReadUser1);
////根据场景 -- 更新已阅用户记录
//        JSONObject toReadUser2 = new JSONObject();
//        toReadUser2.put("toReadUserId", "admin");
//        toReadUser2.put("toReadUserName", "超级管理员");
//        toReadUser2.put("Status", "End");
//        toReadUser2.put("remark", "已读");
//        toReadUser2.put("startTime", "2022-12-09 17:34:46");
//        toReadUser2.put("readTime", "2022-12-10 17:34:46");
//        toReadUsers.add(toReadUser2);
//        toReadInfoDTO.setToReadUser(toReadUsers);
////        toReadInfoDTO.setOrunId("a18a585c07b1c04cb00b66e0b76ca9125456");
//        toReadInfoDTO.setNodeId("T10005");
//        toReadInfoDTO.setUrl("https://portal.lysky.com:6205/bpmui/#/formRender/run?docUnid=79ec0af89702d641d839d5bbb5e9073b0e4b");
//        toReadInfoDTO.setMobileUrl("https://portal.lysky.com:6205/bpmui/#/formRender/run?docUnid=79ec0af89702d641d839d5bbb5e9073b0e4b");
//        toReadInfoDTO.setNodeName("部门审批");
//    }
}